from nonebot import on_shell_command
from nonebot.adapters.cqhttp import Bot, MessageEvent
from nonebot.rule import ArgumentParser
from nonebot.typing import T_State

from .handler import *


alias_parser = ArgumentParser()
alias_parser.add_argument('-p', '--print', action='store_true')
alias_parser.add_argument('-gl', '--globally', action='store_true')
alias_parser.add_argument('names', nargs='*')

alias = on_shell_command('alias', parser=alias_parser, priority=10)

unalias_parser = ArgumentParser()
unalias_parser.add_argument('-a', '--all', action='store_true')
unalias_parser.add_argument('-gl', '--globally', action='store_true')
unalias_parser.add_argument('names', nargs='*')

unalias = on_shell_command('unalias', parser=unalias_parser, priority=10)


@alias.handle()
async def _(bot: Bot, event: MessageEvent, state: T_State):
    args = state['args']
    gl = args.globally
    id = 'global' if gl else get_id(event)
    word = '全局别名' if gl else '别名'

    if args.print:
        message = '全局别名：' if gl else ''
        alias_all = aliases.get_alias_all(id)
        for name in sorted(alias_all):
            message += f"\n{name}='{alias_all[name]}'"
        if not gl:
            alias_all_gl = aliases.get_alias_all('global')
            if alias_all_gl:
                message += '\n全局别名：'
                for name in sorted(alias_all_gl):
                    message += f"\n{name}='{alias_all_gl[name]}'"
        message = message.strip()
        if message:
            await alias.finish(message)
        else:
            await alias.finish(f'尚未添加任何{word}')

    message = ''
    names = args.names
    for name in names:
        if '=' in name:
            name, command = name.split('=', 1)
            if name and command and aliases.add_alias(id, name, command):
                message += f"成功添加{word}：{name}='{command}'\n"
        else:
            command = aliases.get_alias(id, name)
            if command:
                message += f"{name}='{command}'\n"
            else:
                message += f"不存在的{word}：{name}\n"

    message = message.strip()
    if message:
        await alias.send(message)


@unalias.handle()
async def _(bot: Bot, event: MessageEvent, state: T_State):
    args = state['args']
    gl = args.globally
    id = 'global' if gl else get_id(event)
    word = '全局别名' if gl else '别名'

    if args.all:
        if aliases.del_alias_all(id):
            await unalias.finish('成功删除所有{word}')

    message = ''
    names = args.names
    for name in names:
        if aliases.get_alias(id, name):
            if aliases.del_alias(id, name):
                message += f"成功删除{word}：{name}\n"
        else:
            message += f"不存在的{word}：{name}\n"

    message = message.strip()
    if message:
        await unalias.send(message)



import json
from pathlib import Path

data_path = Path('data/alias')
if not data_path.exists():
    data_path.mkdir(parents=True)


class AliasList():
    def __init__(self, path: Path):
        self.path = path
        self.list = self._load_alias()

    def _load_alias(self) -> dict:
        if self.path.exists():
            return json.load(self.path.open('r', encoding='utf-8'))
        else:
            return {}

    def _dump_alias(self) -> bool:
        json.dump(
            self.list,
            self.path.open('w', encoding='utf-8'),
            indent=4,
            separators=(',', ': '),
            ensure_ascii=False
        )
        return True

    def add_alias(self, id: str, name: str, command: str) -> bool:
        if id not in self.list:
            self.list[id] = {}
        self.list[id][name] = command
        return self._dump_alias()

    def del_alias(self, id: str, name: str) -> bool:
        if id not in self.list:
            return False
        self.list[id].pop(name)
        return self._dump_alias()

    def del_alias_all(self, id: str) -> bool:
        self.list[id] = {}
        return self._dump_alias()

    def get_alias(self, id: str, name: str) -> str:
        if id not in self.list:
            return ''
        if name not in self.list[id]:
            return ''
        return self.list[id][name]

    def get_alias_all(self, id: str) -> dict:
        if id not in self.list:
            return {}
        return self.list[id].copy()


aliases = AliasList(data_path / 'aliases.json')



import re
from nonebot.adapters.cqhttp import Bot, Event, MessageEvent, GroupMessageEvent
from nonebot.message import event_preprocessor
from nonebot.typing import T_State

from .alias_list import aliases


@event_preprocessor
async def _(bot: Bot, event: Event, state: T_State):
    if not isinstance(event, MessageEvent):
        return
    msgs = event.get_message()
    if len(msgs) < 1 or msgs[0].type != 'text':
        return
    msg = str(msgs[0]).lstrip()
    if not msg:
        return
    alias_all = aliases.get_alias_all(get_id(event))
    alias_all_gl = aliases.get_alias_all('global')
    alias_all_gl.update(alias_all)
    for name in sorted(alias_all_gl, reverse=True):
        if re.match(f'{name}', msg):
            msg = re.sub(f'{name}', f'{alias_all_gl[name]}', msg, count=1)
            event.message[0].data['text'] = msg
            return


def get_id(event: MessageEvent):
    if isinstance(event, GroupMessageEvent):
        return 'group_' + str(event.group_id)
    else:
        return 'private_' + str(event.user_id)
